from django.conf import settings
from django.db import models
from . import exceptions

def get_mappings():
    '''
    Get all mappings from the settings
    
    To use the Django Tags Input module the `TAGS_INPUT_SETTINGS` must be
    defined.
    '''
    return getattr(settings, 'TAGS_INPUT_MAPPINGS', {})

def get_mapping(model_or_queryset):
    '''Get the mapping for a given model or queryset'''
    mappings = get_mappings()

    if isinstance(model_or_queryset, models.query.QuerySet):
        queryset = model_or_queryset
        model = model_or_queryset.model
    elif issubclass(model_or_queryset, models.Model):
        queryset = model_or_queryset.objects.all()
        model = model_or_queryset
    else:
        raise TypeError('Only `django.db.model.Model` and '
            '`django.db.query.QuerySet` objects are valid arguments')

    meta = model._meta
    mapping_key = meta.app_label + '.' + meta.object_name

    mapping = mappings.get(mapping_key)
    if mapping:
        mapping = mapping.copy()
    else:
        raise exceptions.MappingUndefined('Unable to find mapping '
            'for %s' % mapping_key)

    mapping['app'] = meta.app_label
    mapping['model'] = meta.object_name
    mapping['queryset'] = queryset

    return mapping.copy()




from django.db import models
from django import http
from django.utils import simplejson
from . import utils

def autocomplete(request, app, model, field):
    model = models.get_model(app, model)
    mapping = utils.get_mapping(model)

    queryset = (mapping['queryset']
        .values_list(field, flat=True)
        .order_by(*mapping.get('ordering', [field]))
    )

    term = request.GET.get('term')
    if term:
        queryset = queryset.filter(**{'%s__istartswith' % field: term})

    max_results = request.GET.get('max_results')
    if max_results and max_results.isdigit():
        max_results = int(max_results)
    else:
        max_results = 10

    results = list(queryset[:max_results])
    if results:
        response = simplejson.dumps(results),
    else:
        response = ''

    return http.HttpResponse(response, mimetype='application/javascript')




from django.conf import settings
from django import forms
from django.template.loader import render_to_string
from django.core import urlresolvers
from django.contrib.admin import widgets


class TagsInputWidget(forms.SelectMultiple):
    def __init__(self, *args, **kwargs):
        forms.SelectMultiple.__init__(self, *args, **kwargs)

    def render(self, name, value, attrs=None, choices=()):
        context = self.build_attrs(attrs, name=name)
        context['STATIC_URL'] = settings.STATIC_URL
        context['mapping'] = self.mapping
        context['autocomplete_url'] = urlresolvers.reverse(
            'tags_input:autocomplete',
            kwargs=dict(
                app=self.mapping['app'],
                model=self.mapping['model'],
                field=self.mapping['field'],
            ),
        )

        if value:
            field = self.mapping['field']

            ids = []
            for v in value:
                if isinstance(v, (int, long)):
                    ids.append(v)

            values_map = dict(self.mapping['queryset']
                .filter(pk__in=ids)
                .values_list('pk', field)
            )

            values = []
            for v in value:
                if isinstance(v, (int, long)):
                    values.append(values_map[v])
                else:
                    values.append(v)

            context['values'] = ', '.join(values)

        return render_to_string('tags_input_widget.html', context)

    def value_from_datadict(self, data, files, name):
        tags = data.get(name).split(',')
        incomplete = data.get(name + '_incomplete')
        if incomplete != data.get(name + '_default'):
            tags += incomplete.split(',')
        return [t.strip() for t in tags if t]
    
    class Media:
        css = {
            'all': (
                'css/jquery.tagsinput.css',
                'css/base/jquery.ui.all.css',
            ),
        }
        js = (
            'js/jquery-1.7.2.min.js',
            'js/jquery-ui-18.1.16.min.js',
            'js/jquery.tagsinput.js',
        )

class AdminTagsInputWidget(widgets.FilteredSelectMultiple, TagsInputWidget):
    pass




from django.conf.urls import patterns, url

urlpatterns = patterns('tags_input.views',
    url(r'^autocomplete/(?P<app>\w+)/(?P<model>\w+)/(?P<field>\w+)/$',
        'autocomplete', name='autocomplete'),
)




__name__ = 'django-tags-input'
__version__ = '1.3'
__author__ = 'Rick van Hattem'
__author_email__ = 'Rick.van.Hattem@Fawo.nl'
__description__ = ('''Django jQuery Tags Input is a Django field and widget'''
    '''wrapper that uses `django-selectable for autocompletion''')
__url__ = 'https://github.com/WoLpH/django-tags-input'




from django import forms
from . import widgets
from . import utils
from django.core.exceptions import ValidationError

class TagsInputField(forms.ModelMultipleChoiceField):
    widget = widgets.TagsInputWidget

    def __init__(self, queryset, **kwargs):
        super(TagsInputField, self).__init__(queryset, **kwargs)
        self.create_missing = kwargs.pop('create_missing', False)
        self.mapping = kwargs.pop('mapping', None)
        self.widget.mapping = self.get_mapping()

    def get_mapping(self):
        if not self.mapping:
            self.mapping = mapping = utils.get_mapping(self.queryset)
            mapping['queryset'] = self.queryset
            mapping['create_missing'] = (self.create_missing or 
                mapping.get('create_missing', False))

        return self.mapping

    def clean(self, value):
        mapping = self.get_mapping()
        field = mapping['field']
        values = dict(self.queryset
            .filter(**{'%s__in' % field: value})
            .values_list(field, 'pk')
        )
        missing = set(value) - set(values)
        if missing:
            if mapping['create_missing']:
                for v in missing:
                    o = self.queryset.model(**{
                        field: v,
                    })
                    if hasattr(o, 'clean'):
                        o.clean()
                    elif hasattr(o, 'full_clean'):
                        o.full_clean()
                    o.save()
                    values[v] = o.pk
            else:
                raise ValidationError(self.error_messages['invalid_choice']
                    % ', '.join(missing))

        ids = []
        for v in value:
            ids.append(values[v])

        return forms.ModelMultipleChoiceField.clean(self, ids)

class AdminTagsInputField(TagsInputField):
    widget = widgets.AdminTagsInputWidget




import tags_input
import os
from setuptools import setup

if os.path.isfile('README.rst'):
    long_description = open('README.rst').read()
else:
    long_description = ('See http://pypi.python.org/pypi/'
        + tags_input.__name__)

setup(
    name=tags_input.__name__,
    version=tags_input.__version__,
    author=tags_input.__author__,
    author_email=tags_input.__author_email__,
    description=tags_input.__description__,
    url=tags_input.__url__,
    license='BSD',
    packages=['tags_input'],
    package_data={
        'tags_input': [
            'templates/*.html',
            'static/js/*.js',
            'static/css/*.css',
            'static/css/base/*.css',
            'static/css/base/images/*.png',
        ],
    },
    long_description=long_description,
    test_suite='nose.collector',
    setup_requires=['nose'],
    classifiers=[
        'License :: OSI Approved :: BSD License',
    ],
)

